@@ -51,18 +51,7 @@
 
 #include <osmocom/netif/sctp.h>
 
-
-/*! \addtogroup stream Osmocom Stream Socket (client side)
- *  @{
- *
- *  This code is intended to abstract any use of stream-type sockets,
- *  such as TCP and SCTP.  It offers client side implementation,
- * fully integrated with the libosmocore select loop abstraction.
- */
-
-/*! \file stream_cli.c
- *  \brief Osmocom stream socket helpers (client side)
- */
+/*! \file stream_cli.c */
 
 #define LOGSCLI(cli, level, fmt, args...) \
 	LOGP(DLINP, level, "CLICONN(%s,%s){%s} " fmt, \
@@ -114,12 +103,11 @@ struct osmo_stream_cli {
 	int				sk_domain;
 	int				sk_type;
 	uint16_t			proto;
-	int (*connect_cb)(struct osmo_stream_cli *cli);
-	int (*disconnect_cb)(struct osmo_stream_cli *cli);
-	int (*read_cb)(struct osmo_stream_cli *cli);
-	int (*iofd_read_cb)(struct osmo_stream_cli *cli, struct msgb *msg);
-	int (*write_cb)(struct osmo_stream_cli *cli);
-	int (*segmentation_cb)(struct msgb *msg);
+	osmo_stream_cli_connect_cb_t	connect_cb;
+	osmo_stream_cli_disconnect_cb_t	disconnect_cb;
+	osmo_stream_cli_read_cb_t	read_cb;
+	osmo_stream_cli_read_cb2_t	iofd_read_cb;
+	osmo_stream_cli_segmentation_cb_t segmentation_cb;
 	void				*data;
 	int				flags;
 	int				reconnect_timeout;
@@ -128,7 +116,11 @@ struct osmo_stream_cli {
 
 void osmo_stream_cli_close(struct osmo_stream_cli *cli);
 
-/*! \brief Re-connect an Osmocom Stream Client
+/*! \addtogroup stream_cli
+ *  @{
+ */
+
+/*! Re-connect an Osmocom Stream Client.
  *  If re-connection is enabled for this client
  *  (which is the case unless negative timeout was explicitly set via osmo_stream_cli_set_reconnect_timeout() call),
  *  we close any existing connection (if any) and schedule a re-connect timer */
@@ -147,7 +139,7 @@ void osmo_stream_cli_reconnect(struct osmo_stream_cli *cli)
 	osmo_timer_schedule(&cli->timer, cli->reconnect_timeout, 0);
 }
 
-/*! \brief Check if Osmocom Stream Client is in connected state
+/*! Check if Osmocom Stream Client is in connected state.
  *  \param[in] cli Osmocom Stream Client
  *  \return true if connected, false otherwise
  */
@@ -156,6 +148,17 @@ bool osmo_stream_cli_is_connected(struct osmo_stream_cli *cli)
 	return cli->state == STREAM_CLI_STATE_CONNECTED;
 }
 
+/*! Check if Osmocom Stream Client is opened (has an FD available) according to
+ *  its current state.
+ *  \param[in] cli Osmocom Stream Client
+ *  \return true if fd is available (osmo_stream_cli_get_fd()), false otherwise
+ */
+static bool stream_cli_is_opened(const struct osmo_stream_cli *cli)
+{
+	return cli->state == STREAM_CLI_STATE_CONNECTING ||
+	       cli->state == STREAM_CLI_STATE_CONNECTED;
+}
+
 static void osmo_stream_cli_close_iofd(struct osmo_stream_cli *cli)
 {
 	if (!cli->iofd)
@@ -174,7 +177,7 @@ static void osmo_stream_cli_close_ofd(struct osmo_stream_cli *cli)
 	cli->ofd.fd = -1;
 }
 
-/*! \brief Close an Osmocom Stream Client
+/*! Close an Osmocom Stream Client.
  *  \param[in] cli Osmocom Stream Client to be closed
  *  We unregister the socket fd from the osmocom select() loop
  *  abstraction and close the socket */
@@ -211,7 +214,7 @@ void osmo_stream_cli_close(struct osmo_stream_cli *cli)
 	}
 }
 
-/*! \brief Get file descriptor of the stream client socket
+/*! Retrieve file descriptor of the stream client socket.
  *  \param[in] cli Stream Client of which we want to obtain the file descriptor
  *  \returns File descriptor or negative in case of error */
 int
@@ -229,6 +232,18 @@ osmo_stream_cli_get_fd(const struct osmo_stream_cli *cli)
 	return -EINVAL;
 }
 
+/*! Retrieve osmo_io descriptor of the stream client socket.
+ *  This function must not be called on a stream client in legacy osmo_fd mode!
+ *  The iofd is only valid once/after osmo_stream_cli_open() has successfully returned.
+ *  \param[in] cli Stream Client of which we want to obtain the file descriptor
+ *  \returns osmo_io_fd of stream client, or NULL if stream not yet opened. */
+struct osmo_io_fd *
+osmo_stream_cli_get_iofd(const struct osmo_stream_cli *cli)
+{
+	OSMO_ASSERT(cli->mode == OSMO_STREAM_MODE_OSMO_IO);
+	return cli->iofd;
+}
+
 static void osmo_stream_cli_read(struct osmo_stream_cli *cli)
 {
 	LOGSCLI(cli, LOGL_DEBUG, "message received\n");
@@ -404,7 +419,7 @@ static int osmo_stream_cli_fd_cb(struct osmo_fd *ofd, unsigned int what)
 
 static void cli_timer_cb(void *data);
 
-/*! \brief Create an Osmocom stream client
+/*! Create an Osmocom stream client.
  *  \param[in] ctx talloc context from which to allocate memory
  *  This function allocates a new \ref osmo_stream_cli and initializes
  *  it with default values (5s reconnect timer, TCP protocol)
@@ -444,10 +459,23 @@ static void stream_cli_iofd_read_cb(struct osmo_io_fd *iofd, int res, struct msg
 		stream_cli_handle_connecting(cli, res);
 		break;
 	case STREAM_CLI_STATE_CONNECTED:
-		if (res == 0)
+		switch (res) {
+		case -EPIPE:
+		case -ECONNRESET:
+			LOGSCLI(cli, LOGL_ERROR, "lost connection with srv (%d)\n", res);
 			osmo_stream_cli_reconnect(cli);
-		else if (cli->iofd_read_cb)
-			cli->iofd_read_cb(cli, msg);
+			break;
+		case 0:
+			LOGSCLI(cli, LOGL_NOTICE, "connection closed with srv\n");
+			osmo_stream_cli_reconnect(cli);
+			break;
+		default:
+			LOGSCLI(cli, LOGL_DEBUG, "received %d bytes from srv\n", res);
+			break;
+		}
+		/* Notify user of new data or error: */
+		if (cli->iofd_read_cb)
+			cli->iofd_read_cb(cli, res, msg);
 		else
 			msgb_free(msg);
 		break;
@@ -495,11 +523,22 @@ static void stream_cli_iofd_recvmsg_cb(struct osmo_io_fd *iofd, int res, struct
 		stream_cli_handle_connecting(cli, res);
 		break;
 	case STREAM_CLI_STATE_CONNECTED:
-		if (res == 0)
+		switch (res) {
+		case -EPIPE:
+		case -ECONNRESET:
+			LOGSCLI(cli, LOGL_ERROR, "lost connection with srv (%d)\n", res);
 			osmo_stream_cli_reconnect(cli);
-		/* Forward message to read callback, also if the connection failed. */
+			break;
+		case 0:
+			LOGSCLI(cli, LOGL_NOTICE, "connection closed with srv\n");
+			osmo_stream_cli_reconnect(cli);
+			break;
+		default:
+			break;
+		}
+		/* Notify user of new data or error: */
 		if (cli->iofd_read_cb)
-			cli->iofd_read_cb(cli, msg);
+			cli->iofd_read_cb(cli, res, msg);
 		else
 			msgb_free(msg);
 		break;
@@ -517,7 +556,7 @@ static const struct osmo_io_ops osmo_stream_cli_ioops_sctp = {
 #endif
 
 
-/*! \brief Set a name on the cli object (used during logging)
+/*! Set a name on the cli object (used during logging).
  *  \param[in] cli stream_cli whose name is to be set
  *  \param[in] name the name to be set on cli
  */
@@ -528,7 +567,7 @@ void osmo_stream_cli_set_name(struct osmo_stream_cli *cli, const char *name)
 		osmo_iofd_set_name(cli->iofd, name);
 }
 
-/*! \brief Retrieve name previously set on the cli object (see osmo_stream_cli_set_name())
+/*! Retrieve name previously set on the cli object (see osmo_stream_cli_set_name()).
  *  \param[in] cli stream_cli whose name is to be retrieved
  *  \returns The name to be set on cli; NULL if never set
  */
@@ -537,7 +576,8 @@ const char *osmo_stream_cli_get_name(const struct osmo_stream_cli *cli)
 	return cli->name;
 }
 
-/*! \brief Set the remote address to which we connect
+/*! Set the remote address to which we connect.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] addr Remote IP address
  */
@@ -547,8 +587,9 @@ osmo_stream_cli_set_addr(struct osmo_stream_cli *cli, const char *addr)
 	osmo_stream_cli_set_addrs(cli, &addr, 1);
 }
 
-/*! \brief Set the remote address set to which we connect.
+/*! Set the remote address set to which we connect.
  *  Useful for protocols allowing connecting to more than one address (such as SCTP)
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] addr Remote IP address set
  *  \return negative on error, 0 on success
@@ -572,7 +613,8 @@ int osmo_stream_cli_set_addrs(struct osmo_stream_cli *cli, const char **addr, si
 	return 0;
 }
 
-/*! \brief Set the remote port number to which we connect
+/*! Set the remote port number to which we connect.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] port Remote port number
  */
@@ -583,7 +625,8 @@ osmo_stream_cli_set_port(struct osmo_stream_cli *cli, uint16_t port)
 	cli->flags |= OSMO_STREAM_CLI_F_RECONF;
 }
 
-/*! \brief Set the local port number for the socket (to be bound to)
+/*! Set the local port number for the socket (to be bound to).
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] port Local port number
  */
@@ -594,7 +637,8 @@ osmo_stream_cli_set_local_port(struct osmo_stream_cli *cli, uint16_t port)
 	cli->flags |= OSMO_STREAM_CLI_F_RECONF;
 }
 
-/*! \brief Set the local address for the socket (to be bound to)
+/*! Set the local address for the socket (to be bound to).
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] port Local host name
  */
@@ -604,8 +648,9 @@ osmo_stream_cli_set_local_addr(struct osmo_stream_cli *cli, const char *addr)
 	osmo_stream_cli_set_local_addrs(cli, &addr, 1);
 }
 
-/*! \brief Set the local address set to which we connect.
+/*! Set the local address set to which we bind.
  *  Useful for protocols allowing bind to more than one address (such as SCTP)
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] addr Local IP address set
  *  \return negative on error, 0 on success
@@ -629,7 +674,8 @@ int osmo_stream_cli_set_local_addrs(struct osmo_stream_cli *cli, const char **ad
 	return 0;
 }
 
-/*! \brief Set the protocol for the stream client socket
+/*! Set the protocol for the stream client socket.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] proto Protocol (like IPPROTO_TCP (default), IPPROTO_SCTP, ...)
  */
@@ -642,7 +688,7 @@ osmo_stream_cli_set_proto(struct osmo_stream_cli *cli, uint16_t proto)
 
 /* Configure client side segmentation for the iofd */
 static void configure_cli_segmentation_cb(struct osmo_stream_cli *cli,
-					       int (*segmentation_cb)(struct msgb *msg))
+					  osmo_stream_cli_segmentation_cb_t segmentation_cb)
 {
 	/* Copy default settings */
 	struct osmo_io_ops client_ops;
@@ -652,22 +698,23 @@ static void configure_cli_segmentation_cb(struct osmo_stream_cli *cli,
 	osmo_iofd_set_ioops(cli->iofd, &client_ops);
 }
 
-/*! \brief Set the segmentation callback for the client
+/*! Set the segmentation callback for the client.
  *  \param[in,out] cli Stream Client to modify
  *  \param[in] segmentation_cb Target segmentation callback
  */
 void osmo_stream_cli_set_segmentation_cb(struct osmo_stream_cli *cli,
-					 int (*segmentation_cb)(struct msgb *msg))
+					 osmo_stream_cli_segmentation_cb_t segmentation_cb)
 {
 	cli->segmentation_cb = segmentation_cb;
 	if (cli->iofd) /* Otherwise, this will be done in osmo_stream_cli_open() */
 		configure_cli_segmentation_cb(cli, segmentation_cb);
 }
 
-/*! \brief Set the socket type for the stream server link
+/*! Set the socket type for the stream server link.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] type Socket Type (like SOCK_STREAM (default), SOCK_SEQPACKET, ...)
- *  \returns zero on success, negative on error.
+ *  \returns zero on success, negative -errno on error.
  */
 int osmo_stream_cli_set_type(struct osmo_stream_cli *cli, int type)
 {
@@ -683,10 +730,11 @@ int osmo_stream_cli_set_type(struct osmo_stream_cli *cli, int type)
 	return 0;
 }
 
-/*! \brief Set the socket type for the stream server link
+/*! Set the socket type for the stream server link.
+ *  Any changes to this setting will only become active upon next (re)connect.
  *  \param[in] cli Stream Client to modify
  *  \param[in] type Socket Domain (like AF_UNSPEC (default for IP), AF_UNIX, AF_INET, ...)
- *  \returns zero on success, negative on error.
+ *  \returns zero on success, negative -errno on error.
  */
 int osmo_stream_cli_set_domain(struct osmo_stream_cli *cli, int domain)
 {
@@ -704,7 +752,7 @@ int osmo_stream_cli_set_domain(struct osmo_stream_cli *cli, int domain)
 	return 0;
 }
 
-/*! \brief Set the reconnect time of the stream client socket
+/*! Set the reconnect time of the stream client socket.
  *  \param[in] cli Stream Client to modify
  *  \param[in] timeout Re-connect timeout in seconds or negative value to disable auto-reconnection */
 void
@@ -713,7 +761,7 @@ osmo_stream_cli_set_reconnect_timeout(struct osmo_stream_cli *cli, int timeout)
 	cli->reconnect_timeout = timeout;
 }
 
-/*! \brief Set application private data of the stream client socket
+/*! Set application private data of the stream client socket.
  *  \param[in] cli Stream Client to modify
  *  \param[in] data User-specific data (available in call-back functions) */
 void
@@ -722,7 +770,7 @@ osmo_stream_cli_set_data(struct osmo_stream_cli *cli, void *data)
 	cli->data = data;
 }
 
-/*! \brief Get application private data of the stream client socket
+/*! Retrieve application private data of the stream client socket.
  *  \param[in] cli Stream Client to modify
  *  \returns Application private data, as set by \ref osmo_stream_cli_set_data() */
 void *osmo_stream_cli_get_data(struct osmo_stream_cli *cli)
@@ -730,7 +778,9 @@ void *osmo_stream_cli_get_data(struct osmo_stream_cli *cli)
 	return cli->data;
 }
 
-/*! \brief Get the stream client socket description.
+/*! Retrieve the stream client socket description.
+ *  Calling this function will build a string that describes the socket in terms of its local/remote
+ *  address/port.  The returned name is stored in a static buffer; it is hence not re-entrant or thread-safe.
  *  \param[in] cli Stream Client to examine
  *  \returns Socket description or NULL in case of error */
 char *osmo_stream_cli_get_sockname(const struct osmo_stream_cli *cli)
@@ -743,7 +793,8 @@ char *osmo_stream_cli_get_sockname(const struct osmo_stream_cli *cli)
 	return buf;
 }
 
-/*! \brief Get Osmocom File Descriptor of the stream client socket
+/*! Retrieve Osmocom File Descriptor of the stream client socket.
+ *  This function only works in case you operate osmo_stream_cli in osmo_fd mode!
  *  \param[in] cli Stream Client to modify
  *  \returns Pointer to \ref osmo_fd */
 struct osmo_fd *
@@ -753,52 +804,54 @@ osmo_stream_cli_get_ofd(struct osmo_stream_cli *cli)
 	return &cli->ofd;
 }
 
-/*! \brief Set the call-back function called on connect of the stream client socket
+/*! Set the call-back function called on connect of the stream client socket.
+ *  The call-back function registered via this function will be called upon completion of the non-blocking
+ *  outbound connect operation.
  *  \param[in] cli Stream Client to modify
  *  \param[in] connect_cb Call-back function to be called upon connect */
 void
 osmo_stream_cli_set_connect_cb(struct osmo_stream_cli *cli,
-	int (*connect_cb)(struct osmo_stream_cli *cli))
+			       osmo_stream_cli_connect_cb_t connect_cb)
 {
 	cli->connect_cb = connect_cb;
 }
 
-/*! \brief Set the call-back function called on disconnect of the stream client socket
+/*! Set the call-back function called on disconnect of the stream client socket.
  *  \param[in] cli Stream Client to modify
  *  \param[in] disconnect_cb Call-back function to be called upon disconnect */
 void osmo_stream_cli_set_disconnect_cb(struct osmo_stream_cli *cli,
-				       int (*disconnect_cb)(struct osmo_stream_cli *cli))
+				       osmo_stream_cli_disconnect_cb_t disconnect_cb)
 {
 	cli->disconnect_cb = disconnect_cb;
 }
 
-/*! \brief Set the call-back function called to read from the stream client socket
- *  This function will configure osmo_stream_cli to use osmo_ofd internally.
+/*! Set the call-back function called to read from the stream client socket.
+ *  This function will implicitly configure osmo_stream_cli to use legacy osmo_ofd mode.
  *  \param[in] cli Stream Client to modify
  *  \param[in] read_cb Call-back function to be called when we want to read */
 void
 osmo_stream_cli_set_read_cb(struct osmo_stream_cli *cli,
-			    int (*read_cb)(struct osmo_stream_cli *cli))
+			    osmo_stream_cli_read_cb_t read_cb)
 {
 	OSMO_ASSERT(cli->mode != OSMO_STREAM_MODE_OSMO_IO);
 	cli->mode = OSMO_STREAM_MODE_OSMO_FD;
 	cli->read_cb = read_cb;
 }
 
-/*! \brief Set the call-back function called to read from the stream client socket
- *  This function will configure osmo_stream_cli to use osmo_iofd internally.
+/*! Set the call-back function called to read from the stream client socket.
+ *  This function will implicitly configure osmo_stream_cli to use osmo_iofd mode.
  *  \param[in] cli Stream Client to modify
  *  \param[in] read_cb Call-back function to be called when data was read from the socket */
 void
 osmo_stream_cli_set_read_cb2(struct osmo_stream_cli *cli,
-			    int (*read_cb)(struct osmo_stream_cli *cli, struct msgb *msg))
+			     osmo_stream_cli_read_cb2_t read_cb)
 {
 	OSMO_ASSERT(cli->mode != OSMO_STREAM_MODE_OSMO_FD);
 	cli->mode = OSMO_STREAM_MODE_OSMO_IO;
 	cli->iofd_read_cb = read_cb;
 }
 
-/*! \brief Destroy a Osmocom stream client (includes close)
+/*! Destroy a Osmocom stream client (includes close).
  *  \param[in] cli Stream Client to destroy */
 void osmo_stream_cli_destroy(struct osmo_stream_cli *cli)
 {
@@ -808,7 +861,7 @@ void osmo_stream_cli_destroy(struct osmo_stream_cli *cli)
 	talloc_free(cli);
 }
 
-/*! \brief DEPRECATED: use osmo_stream_cli_set_reconnect_timeout() or osmo_stream_cli_reconnect() instead!
+/*! DEPRECATED: use osmo_stream_cli_set_reconnect_timeout() or osmo_stream_cli_reconnect() instead!
  * Open connection of an Osmocom stream client
  *  \param[in] cli Stream Client to connect
  *  \param[in] reconect 1 if we should not automatically reconnect
@@ -866,24 +919,37 @@ error_close_socket:
 	return -EIO;
 }
 
-/*! \brief Set the NODELAY socket option to avoid Nagle-like behavior
+/*! Set the NODELAY socket option to avoid Nagle-like behavior.
  *  Setting this to nodelay=true will automatically set the NODELAY
  *  socket option on any socket established via \ref osmo_stream_cli_open
- *  or any re-connect.  You have to set this _before_ opening the
+ *  or any re-connect. This can be set either before or after opening the
  *  socket.
  *  \param[in] cli Stream client whose sockets are to be configured
  *  \param[in] nodelay whether to set (true) NODELAY before connect()
  */
 void osmo_stream_cli_set_nodelay(struct osmo_stream_cli *cli, bool nodelay)
 {
+	int fd;
 	if (nodelay)
 		cli->flags |= OSMO_STREAM_CLI_F_NODELAY;
 	else
 		cli->flags &= ~OSMO_STREAM_CLI_F_NODELAY;
+
+	if (!stream_cli_is_opened(cli))
+		return; /* Config will be applied upon open() time */
+
+	if ((fd = osmo_stream_cli_get_fd(cli)) < 0) {
+		LOGSCLI(cli, LOGL_ERROR, "set_nodelay(%u): failed obtaining socket\n", nodelay);
+		return;
+	}
+	if (stream_setsockopt_nodelay(fd, cli->proto, nodelay ? 1 : 0) < 0)
+		LOGSCLI(cli, LOGL_ERROR, "set_nodelay(%u): failed setsockopt err=%d\n",
+			nodelay, errno);
 }
 
-/*! \brief Open connection of an Osmocom stream client
- *  By default the client will automatically reconnect after default timeout.
+/*! Open connection of an Osmocom stream client.
+ *  This will initiate an non-blocking outbound connect to the configured destination (server) address.
+ *  By default the client will automatically attempt to reconnect after default timeout.
  *  To disable this, use osmo_stream_cli_set_reconnect_timeout() before calling this function.
  *  \param[in] cli Stream Client to connect
  *  \return negative on error, 0 on success */
@@ -1002,7 +1068,8 @@ static void cli_timer_cb(void *data)
 	osmo_stream_cli_open(cli);
 }
 
-/*! \brief Enqueue data to be sent via an Osmocom stream client
+/*! Enqueue data to be sent via an Osmocom stream client..
+ *  This is the function you use for writing/sending/transmitting data via the osmo_stream_cli.
  *  \param[in] cli Stream Client through which we want to send
  *  \param[in] msg Message buffer to enqueue in transmit queue */
 void osmo_stream_cli_send(struct osmo_stream_cli *cli, struct msgb *msg)
@@ -1038,11 +1105,15 @@ void osmo_stream_cli_send(struct osmo_stream_cli *cli, struct msgb *msg)
 	}
 }
 
-/*! \brief Receive data via an Osmocom stream client
+/*! Receive data via an Osmocom stream client in osmo_fd mode.
  *  \param[in] cli Stream Client through which we want to send
  *  \param msg pre-allocate message buffer to which received data is appended
  *  \returns number of bytes read; <=0 in case of error
  *
+ *  Application programs using the legacy osmo_fd mode of osmo_stream_cli will use
+ *  this function to read/receive from a stream client socket after they have been notified that
+ *  it is readable (via select/poll).
+ *
  *  If conn is an SCTP connection, additional specific considerations shall be taken:
  *  - msg->cb is always filled with SCTP ppid, and SCTP stream values, see msgb_sctp_*() APIs.
  *  - If an SCTP notification was received when reading from the SCTP socket,
@@ -1058,6 +1129,7 @@ int osmo_stream_cli_recv(struct osmo_stream_cli *cli, struct msgb *msg)
 	int ret;
 	OSMO_ASSERT(cli);
 	OSMO_ASSERT(msg);
+	OSMO_ASSERT(cli->mode == OSMO_STREAM_MODE_OSMO_FD);
 
 	switch (cli->sk_domain) {
 	case AF_UNIX:
@@ -1103,6 +1175,8 @@ int osmo_stream_cli_recv(struct osmo_stream_cli *cli, struct msgb *msg)
 	return ret;
 }
 
+/*! Clear the transmit queue of the stream client.
+ *  Calling this function wil clear (delete) any pending, not-yet transmitted data from the transmit queue. */
 void osmo_stream_cli_clear_tx_queue(struct osmo_stream_cli *cli)
 {
 	switch (cli->mode) {
@@ -1121,6 +1195,12 @@ void osmo_stream_cli_clear_tx_queue(struct osmo_stream_cli *cli)
 	}
 }
 
+/*! Set given parameter of stream client to given value.
+ *  \param[in] cli stream client on which to set parameter.
+ *  \param[in] par identifier of the parameter to be set.
+ *  \param[in] val value of the parameter to be set.
+ *  \param[in] val_len length of the parameter value.
+ *  \returns 0 in success; negative -errno on error. */
 int osmo_stream_cli_set_param(struct osmo_stream_cli *cli, enum osmo_stream_cli_param par, void *val, size_t val_len)
 {
 	OSMO_ASSERT(cli);
